<h4 class="major">Work Experience</h4>
{% for experience in work_experience %}
    <h4>{{ experience[0]["company"] }} | {{ experience[0]["role"] }}</h4>
    <!-- Metadata -->
    <strong><em>{{experience[0]["month_started"] | pretty_datetime(experience[0]["year_started"], experience[0]["month_ended"], experience[0]["year_ended"], experience[0]["current"])}}</em></strong>
    <!-- Content -->
    <p>{{ experience[1] | safe}}</p>
{% endfor %}